# -*- coding: utf-8 -*-
"""
Testing the properties (metadata) of a document
"""
# $Id: test_properties.py 6355 2007-09-20 17:16:21Z glenfant $

import unittest
import os
from fixures import *

import openxmllib

class PropertiesTest(unittest.TestCase):
    """Testing querying properties from a document"""

    def setUp(self):
        test_file_path = os.path.join(TEST_FILES_IN, ALL_IN_FILES[0])
        self.doc = openxmllib.openXmlDocument(test_file_path)
        return

    def test_coreprops(self):
        """Core properties"""

        props = self.doc.coreProperties

        # We check only some props
        expected = {
            'language': 'fr-FR',
            'title': 'The title of the document'
            }
        for k, v in expected.items():
            self.failUnlessEqual(props[k], v)
        return

    def test_extprops(self):
        """Extended properties"""

        props = self.doc.extendedProperties

        # We check only some props
        expected = {
            'Pages': '1',
            'Words': '18',
            'Paragraphs': '1'}
        for k, v in expected.items():
            self.failUnlessEqual(props[k], v)
        return

    def test_customprops(self):
        """Custom properties"""

        props = self.doc.customProperties
        expected = {
            'custom_title_1': 'custom_value_1',
            'custom_title_2': 'custom_value_2',
            'custom_title_3': 'custom_value_3',
            'custom_title_4': 'custom_value_4',
            }
        for k, v in expected.items():
            self.failUnlessEqual(props[k], v)
        return
# /class PropertiesTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PropertiesTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



# -*- coding: utf-8 -*-
"""
The wordprocessing module handles a WordprocessingML Open XML document (read *.docx)
"""
# $Id: wordprocessing.py 6800 2007-12-04 11:17:01Z glenfant $

import document
from utils import IndexableTextExtractor
import contenttypes as ct


class WordprocessingDocument(document.Document):
    """Handles specific features of a WordprocessingML document"""

    _extpattern_to_mime = {
        '*.docx': ct.CT_WORDPROC_DOCX_PUBLIC,
        '*.docm': ct.CT_WORDPROC_DOCM_PUBLIC,
        '*.dotx': ct.CT_WORDPROC_DOTX_PUBLIC,
        '*.dotm': ct.CT_WORDPROC_DOTM_PUBLIC
        }

    _text_extractors = (
        IndexableTextExtractor(ct.CT_WORDPROC_DOCUMENT, 'wordprocessing-main:t'),
        )

    pass



# -*- coding: utf-8 -*-
"""
Namespaces that may be used in various XML files
"""
# $Id: namespaces.py 6800 2007-12-04 11:17:01Z glenfant $

CONTENT_TYPES = 'http://schemas.openxmlformats.org/package/2006/content-types'

# Properties (common for all openxml types)
CORE_PROPERTIES = 'http://schemas.openxmlformats.org/package/2006/metadata/core-properties'
EXTENDED_PROPERTIES = 'http://schemas.openxmlformats.org/officeDocument/2006/extended-properties'
CUSTOM_PROPERTIES = 'http://schemas.openxmlformats.org/officeDocument/2006/custom-properties'

# Wordprocessing
WP_MAIN = 'http://schemas.openxmlformats.org/wordprocessingml/2006/main'

# Spreadsheet
SS_MAIN = 'http://schemas.openxmlformats.org/spreadsheetml/2006/main'

# Presentation
PR_MAIN = 'http://schemas.openxmlformats.org/drawingml/2006/main'

# Namespaces mapping useful for XPath expression shortcuts
ns_map = {
    'content-types': CONTENT_TYPES,
    'core-properties': CORE_PROPERTIES,
    'extended-properties': EXTENDED_PROPERTIES,
    'custom-properties': CUSTOM_PROPERTIES,
    'wordprocessing-main': WP_MAIN,
    'spreadsheet-main': SS_MAIN,
    'presentation-main': PR_MAIN
    }



# -*- coding: utf-8 -*-
"""
Testing PresentationDocument
"""
# $Id$

import unittest
import os
from fixures import *

import openxmllib

class PresentationTest(unittest.TestCase):
    """Testing querying properties from a document"""

    def setUp(self):
        test_file_path = os.path.join(TEST_FILES_IN, ALL_IN_FILES[2])
        self.doc = openxmllib.openXmlDocument(test_file_path)
        return


    def test_indexableText(self):
        """Indexable text with properties"""

        itext = self.doc.indexableText()
        some_words = (u'Chapter', u'presentation', u'proud', u'three', u'two',
             u'four', u'item', u'one', u'My')
        some_words += (u'false', u'Microsoft Office PowerPoint')
        for word in some_words:
            self.failUnless(word in itext, "%s was expected" % word)
        return


    def test_indexableTextNoprop(self):
        """Indexable text without properties"""

        itext = self.doc.indexableText(include_properties=False)
        some_words = (u'Chapter', u'presentation', u'proud', u'three', u'two',
             u'four', u'item', u'one', u'My')
        for word in some_words:
            self.failUnless(word in itext, "%s was expected" % word)
        return
# /class WordProcessingTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PresentationTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



# -*- coding: utf-8 -*-
"""
Testing the utils module resources
"""
# $Id: test_utils.py 6796 2007-12-04 10:52:51Z glenfant $

import unittest
import os
from fixures import *

import openxmllib

class UtilsTest(unittest.TestCase):
    """Testing resources from the utilsmodule"""


    def test_xmlfile(self):
        """Working around absence of BOM support in lxml"""

        from lxml import etree
        test_file_path = os.path.join(TEST_FILES_IN, ALL_IN_FILES[0])
        doc = openxmllib.openXmlDocument(test_file_path)
        toc_path = os.path.join(doc._cache_dir, '[Content_Types].xml')
        fh = openxmllib.utils.xmlFile(toc_path, 'rb')
        xml = etree.parse(fh)
        self.failUnless(xml.getroot(), "Expected an XML element")
        return

    def test_tounicode(self):
        """Unicodizing an object"""
        toUnicode = openxmllib.utils.toUnicode

        # Non text object
        self.failUnlessEqual(toUnicode(AttributeError), AttributeError)

        # ASCII stuff
        self.failUnlessEqual(toUnicode('foo'), u'foo')

        # Unicode stuff
        self.failUnlessEqual(toUnicode(u'foo'), u'foo')
        return

    def test_itextractor(self):
        """IndexableTextExtractor"""
        IndexableTextExtractor = openxmllib.utils.IndexableTextExtractor
        from StringIO import StringIO
        from openxmllib.contenttypes import CT_WORDPROC_DOCUMENT
        from lxml import etree

        ite = IndexableTextExtractor(CT_WORDPROC_DOCUMENT, 'wordprocessing-main:t')
        indexables = ite.indexableText(etree.parse(StringIO(WP_MAIN_XML)))
        some_words = (u'A', u'full', u'chàractèrs', u'non')
        for word in some_words:
            self.failUnless(word in indexables,
                            "%s expected from %s" % (word, indexables))
        return

# /class UtilsTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(UtilsTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())

###
## Test data
###

# Wordprocessing main document
# application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml


WP_MAIN_XML = """<?xml version="1.0" encoding="utf-8"?>
<w:document xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
            xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
  <w:body>
    <w:p>
      <w:pPr>
        <w:pStyle w:val="Standard" />
        <w:widowControl w:val="on" />
        <w:suppressAutoHyphens w:val="true" />
        <w:ind w:left="0" w:right="0" w:hanging="0" />
        <w:rPr><w:lang w:val="fr-FR" />
        </w:rPr>
      </w:pPr>
      <w:r>
        <w:rPr>
          <w:lang w:val="fr-FR" />
        </w:rPr>
        <w:t xml:space="preserve"
             xmlns:pxs="urn:cleverage:xmlns:post-processings:extra-spaces">A simple example of a .docx document </w:t>
      </w:r>
      <w:r>
        <w:rPr>
          <w:b w:val="on" />
          <w:lang w:val="fr-FR" />
        </w:rPr>
        <w:t xml:space="preserve"
             xmlns:pxs="urn:cleverage:xmlns:post-processings:extra-spaces">with</w:t>
      </w:r><w:r><w:rPr><w:lang w:val="fr-FR" />
        </w:rPr><w:t xml:space="preserve"
                     xmlns:pxs="urn:cleverage:xmlns:post-processings:extra-spaces"> a full set of metadata. And some non ASCII chàractèrs.</w:t>
      </w:r>
    </w:p>
    <w:p>
      <w:pPr>
        <w:pStyle w:val="Standard" />
        <w:widowControl w:val="on" />
        <w:suppressAutoHyphens w:val="true" />
        <w:ind w:left="0" w:right="0" w:hanging="0" />
        <w:rPr>
          <w:lang w:val="fr-FR" />
        </w:rPr>
      </w:pPr>
    </w:p>
    <w:sectPr>
      <w:footnotePr>
        <w:pos w:val="pageBottom" />
        <w:numFmt w:val="decimal" />
        <w:numStart w:val="1" />
        <w:numRestart w:val="continuous" />
      </w:footnotePr>
      <w:endnotePr>
        <w:pos w:val="docEnd" />
        <w:numFmt w:val="lowerRoman" />
        <w:numStart w:val="1" />
      </w:endnotePr>
      <w:pgSz w:orient="portrait" w:w="11905" w:h="16837" />
      <w:pgMar w:top="1134"
               w:left="1134"
               w:bottom="1134"
               w:right="1134"
               w:header="1134"
               w:footer="1134"
               w:gutter="0" />
      <w:cols w:num="0" w:space="0" />
    </w:sectPr>
  </w:body>
</w:document>
"""



# -*- coding: utf-8 -*-
"""
The spreadsheet module handles a SpreadsheetML Open XML document (read *.xlsx)
"""
# $Id: spreadsheet.py 6800 2007-12-04 11:17:01Z glenfant $

import document
from utils import IndexableTextExtractor
import contenttypes as ct


class SpreadsheetDocument(document.Document):
    """Handles specific features of a SpreadsheetML document"""

    _extpattern_to_mime = {
        '*.xlsx': ct.CT_SPREADSHEET_XLSX_PUBLIC,
        '*.xlsm': ct.CT_SPREADSHEET_XLSM_PUBLIC,
        '*.xltx': ct.CT_SPREADSHEET_XLTX_PUBLIC,
        '*.xltm': ct.CT_SPREADSHEET_XLTM_PUBLIC,
        # FIXME: note sure we can honour below types...
#        '*.xlam': ct.CT_SPREADSHEET_XLAM_PUBLIC,
#        '*.xlsb': ct.CT_SPREADSHEET_XLSB_PUBLIC
        }

    _text_extractors = (
        IndexableTextExtractor(ct.CT_SPREADSHEET_SHAREDSTRINGS, 'spreadsheet-main:t'),
        )

    pass



# -*- coding: utf-8 -*-
"""
Environment fixures (stubs, data, ...) for unit testing
"""
# $Id: fixures.py 6796 2007-12-04 10:52:51Z glenfant $

import os
import sys

# Fixing the sys.path such "openxmllib" is on top
this_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(this_dir)
if parent_dir not in sys.path:
    sys.path.insert(0, parent_dir)

# Unit tests notification
if not os.getenv('PYTHON_UNIT_TEST', False):
    os.putenv('PYTHON_UNIT_TEST', 'OK')

# Directories with test files
TEST_FILES_IN = os.path.join(this_dir, 'in')
ALL_IN_FILES = ('wordprocessing1.docx', 'spreadsheet1.xlsx', 'presentation1.pptx')

__all__ = ('TEST_FILES_IN', 'ALL_IN_FILES')



# -*- coding: utf-8 -*-
"""
Testing construction on valid and crappy files
"""
# $Id: test_validfiles.py 6355 2007-09-20 17:16:21Z glenfant $

import unittest
import os
from fixures import *

import openxmllib

class DocumentConstructTest(unittest.TestCase):
    """Testing construction of Document various ways"""


    def setUp(self):
        self.test_file_path = os.path.join(TEST_FILES_IN, ALL_IN_FILES[0])
        return


    def test_file(self):
        """Construction from a file object"""

        fh = file(self.test_file_path, 'rb')
        doc = openxmllib.document.Document(fh)
        self._assertCreation(doc._cache_dir)
        return

    def test_path(self):
        """Construction from a file path"""

        doc = openxmllib.document.Document(self.test_file_path)
        self._assertCreation(doc._cache_dir)
        return

    def test_body(self):
        """Construction from a file content"""

        data = file(self.test_file_path, 'rb').read()
        doc = openxmllib.document.Document(data)
        self._assertCreation(doc._cache_dir)
        return

    def _assertCreation(self, path):
        self.failUnless(os.path.isfile(os.path.join(path, '[Content_Types].xml')),
                        "Expected to have data in path %s" % path)
        return

# /class DocumentConstructTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DocumentConstructTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



# -*- coding: utf-8 -*-
"""
The document modules handles an Open XML document
"""
# $Id: document.py 6800 2007-12-04 11:17:01Z glenfant $

import os
import cStringIO
import tempfile
import zipfile
import shutil
import types
import fnmatch

import lxml

import contenttypes
from namespaces import ns_map
from utils import xmlFile
from utils import toUnicode


class Document(object):
    """Handling of Open XML document (all types)"""

    # These properties must be overriden by subclasses
    _extpattern_to_mime = {} # ({'*.ext': 'aplication/xxx'}, ...}
    _text_extractors = []

    def __init__(self, path_or_file_or_data, mime_type=None):
        """
        @param path_or_file_or_data: ... to the document
        @type path_or_file_or_data: a file path, a file object or a binary buffer
        A file must be opened in 'rb' mode
        """

        self.mime_type = mime_type

        # Some shortcuts
        op_sep = os.path.sep
        op_join = os.path.join
        op_isdir = os.path.isdir
        op_dirname = os.path.dirname

        # Preliminary settings depending on input
        if type(path_or_file_or_data) in types.StringTypes:
            # Path or data
            try:
                if os.path.isfile(path_or_file_or_data):
                    in_file = file(path_or_file_or_data, 'rb')
                    self.filename = os.path.basename(path_or_file_or_data)
                else:
                    raise TypeError
            except:
                in_file = cStringIO.StringIO(path_or_file_or_data)
                self.filename = None
        elif hasattr(path_or_file_or_data, 'read'):
            # File like object (supposed to)
            in_file = path_or_file_or_data
            try:
                self.filename = path_or_file_or_data.name
            except AttributeError, e:
                # A StringIO  or other unnamed stream
                self.filename = None
        else:
            raise TypeError, "Cannot use % documents" % str(path_or_file_or_data.__class__)

        # Inflating the file
        self._cache_dir = tempfile.mkdtemp()
        openxmldoc = zipfile.ZipFile(in_file, 'r', zipfile.ZIP_DEFLATED)
        for outpath in openxmldoc.namelist():
            # We need to be sure that target dir exists
            rel_outpath = op_sep.join(outpath.split('/'))
            abs_outpath = op_join(self._cache_dir, rel_outpath)
            abs_outdir = op_dirname(abs_outpath)
            if not op_isdir(abs_outdir):
                os.makedirs(abs_outdir)
            fh = file(abs_outpath, 'wb')
            fh.write(openxmldoc.read(outpath))
            fh.close()
        openxmldoc.close()
        in_file.close()

        # Getting the content types decl
        ct_file = op_join(self._cache_dir, '[Content_Types].xml')
        self.content_types = contenttypes.ContentTypes(xmlFile(ct_file, 'rb'))
        return


    @property
    def mimeType(self):
        """The official MIME type for this document
        @return: 'application/xxx' for this file
        """

        if self.mime_type:
            # Supposed validated by the factory
            return self.mime_type
        for pattern, mime_type in self._extpattern_to_mime.items():
            if fnmatch.fnmatch(self.filename, pattern):
                return mime_type
        return None


    @property
    def coreProperties(self):
        """Document core properties
        @return: mapping of metadata
        """

        return self._tagValuedProperties(contenttypes.CT_CORE_PROPS)


    @property
    def extendedProperties(self):
        """Document extended properties
        @return: mapping of metadata
        """

        return self._tagValuedProperties(contenttypes.CT_EXT_PROPS)


    def _tagValuedProperties(self, content_type):
        """Document properties for property files having constructs like
         <ns:name>value</ns:name>
         @param content_type: contenttypes.CT_CORE_PROPS or contenttypes.CT_EXT_PROPS
         @return: mapping like {'property name': 'property value', ...}
        """

        rval = {}
        if not content_type in self.content_types.listMetaContentTypes:
            # We fail silently
            return rval
        for tree in self.content_types.getTreesFor(self, content_type):
            for elt in tree.getroot().getchildren():
                tag = elt.tag.split('}')[-1] # Removing namespace if any
                rval[toUnicode(tag)] = toUnicode(elt.text)
        return rval


    @property
    def customProperties(self):
        """Document custom properties
        @return: mapping of metadata
        FIXME: This is ugly. We do not convert the properties as indicated
        with the http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes
        namespace
        """

        rval = {}
        if len(self.content_types.getPathsForContentType(contenttypes.CT_CUSTOM_PROPS)) == 0:
            # We may have no custom properties at all.
            return rval
        XPath = lxml.etree.XPath # Class shortcut
        properties_xpath = XPath('custom-properties:property', ns_map)
        propname_xpath = XPath('@name')
        propvalue_xpath = XPath('*/text()')
        for tree in self.content_types.getTreesFor(self, contenttypes.CT_CUSTOM_PROPS):
            for elt in properties_xpath(tree.getroot()):
                rval[toUnicode(propname_xpath(elt)[0])] = u" ".join(propvalue_xpath(elt))
        return rval


    @property
    def allProperties(self):
        """Helper that merges core, extended and custom properties
        @return: mapping of metadata
        """

        rval = {}
        rval.update(self.coreProperties)
        rval.update(self.extendedProperties)
        rval.update(self.customProperties)
        return rval


    def indexableText(self, include_properties=True):
        """
        Note that self._text_extractors must be overriden by subclasses
        """

        text = set()
        for extractor in self._text_extractors:
            for tree in self.content_types.getTreesFor(self, extractor.content_type):
                words = extractor.indexableText(tree)
                text |= words
        if include_properties:
            for prop_value in self.allProperties.values():
                if prop_value is not None:
                    text.add(prop_value)
        return u' '.join([word for word in text])


    def __del__(self):
        """Cleanup at Document object detetion"""

        self._cleanup()
        return


    def _cleanup(self):
        """Removing all temporary files
        Be warned that "cleanuping" your document makes it unusable.
        """

        shutil.rmtree(self._cache_dir, ignore_errors=True)
        return


    @classmethod
    def canProcessMime(cls, mime_type):
        """Check if we can process such mime type
        @param mime_type: Mime type as 'application/xxx'
        @return: True if we can process such mime
        """

        supported_mimes = cls._extpattern_to_mime.values()
        return mime_type in supported_mimes


    @classmethod
    def canProcessFilename(cls, filename):
        """Check if we can process such file based on name
        @param filename: File name as 'mydoc.docx'
        @return: True if we can process such file
        """

        supported_patterns = cls._extpattern_to_mime.keys()
        for pattern in supported_patterns:
            if fnmatch.fnmatch(filename, pattern):
                return True
        return False




#! python
# -*- coding: utf-8 -*-
"""Command line tool"""

import os
import optparse
import time
import codecs
import locale
import types
import openxmllib

DEFAULT_CHARSET = locale.getpreferredencoding()
USAGE = """%%prog [options] command file
Version: %s
%s
Commands:
* `metadata' shows the file's metadata.
* `words' shows all words from file."""
VERSION = openxmllib.version


class Application(object):
    """Command line utility showing openxml document informations."""

    def __init__(self):

        def check_charset_option(option, opt_str, value, parser):
            """Value must be a valid charset"""
            try:
                dummy = codecs.lookup(value)
            except LookupError, e:
                raise optparse.OptionValueError("Charset '%s' in unknown or not supported by your sytem." % value)
            setattr(parser.values, option.dest, value)
            return

        parser = optparse.OptionParser(usage=USAGE % (VERSION, self.__class__.__doc__),
                                       version=VERSION)
        parser.add_option('-c', '--charset', dest='charset', default=DEFAULT_CHARSET,
                          type='string', action='callback', callback=check_charset_option,
                          help="Converts output to this charset (default %s)" % DEFAULT_CHARSET)
        parser.add_option('-v', '--verbosity', dest='verbosity', default=0, action='count',
                           help="Adds verbosity for each '-v'")
        self.options, self.args = parser.parse_args()
        if (len(self.args) < 2
            or self.args[0] not in self.commands.keys()):
            parser.error("Invalid arguments")
        self.filenames = self.args[1:]
        return

    def run(self):
        self.commands[self.args[0]](self)
        return

    def metadataCmd(self):
        self.log(1, "Showing metadata of %s.", ", ".join(self.filenames))
        for filename in self.filenames:
            self.showMetadata(filename)
        return

    def wordsCmd(self):
        self.log(1, "Showing words of %s.", ", ".join(self.filenames))
        for filename in self.filenames:
            self.showWords(filename)
        return

    commands = {
        'metadata': metadataCmd,
        'words': wordsCmd
        }

    def showMetadata(self, filename):
        if not self.checkfile(filename):
            return
        self.log(1, "Processing %s...", filename)
        doc = openxmllib.openXmlDocument(filename)
        self.log(2, "Core properties:")
        for k, v in doc.coreProperties.items():
            print "%s: %s" % (self.recode(k), self.recode(v))
        self.log(2, "Extended properties:")
        for k, v in doc.extendedProperties.items():
            print "%s: %s" % (self.recode(k), self.recode(v))
        self.log(2, "Custom properties:")
        for k, v in doc.customProperties.items():
            print "%s: %s" % (self.recode(k), self.recode(v))
        return

    def showWords(self, filename):
        if not self.checkfile(filename):
            return
        self.log(1, "Processing %s...", filename)
        start_time = time.time()
        doc = openxmllib.openXmlDocument(filename)
        text = doc.indexableText(include_properties=False)
        duration = time.time() - start_time
        print self.recode(text)
        self.log(1, "Words extracted in %s second(s)", duration)
        return

    def checkfile(self, filename):
        if not os.path.isfile(filename):
            self.log(0, "'%s' is not a file, skipped", filename)
            return False
        return True

    def log(self, required_verbosity, message, *args):
        if self.options.verbosity >= required_verbosity:
            print message % args
        return

    def recode(self, utext):
        if type(utext) is types.UnicodeType:
            return utext.encode(self.options.charset, 'replace')
        return utext

if __name__ == '__main__':
    Application().run()



# -*- coding: utf-8 -*-
"""
Various utilities for openxmllib
"""
# $Id: utils.py 6800 2007-12-04 11:17:01Z glenfant $

import re
from lxml import etree

from namespaces import ns_map

def xmlFile(path, mode='r'):
    """lxml cannot parse XML files starting with a BOM
    (see http://www.w3.org/TR/2000/REC-xml-20001006 in F.1.)
    In case such XML file is used, we must skip these characters
    So we open all XML files for read with 'xmlFile'.
    TODO: File this issue to lxml ML or tracker (feature or bug ?)
    """

    fh = file(path, mode)
    while fh.read(1) != '<': # Ignoring everything before '<?xml...'
        pass
    fh.seek(-1, 1)
    return fh


def toUnicode(objekt):
    """Safely converts anything returned by lxml services to unicode
    @param objekt: anything
    @return: the object itself if not a string, otherwise the unicode of the string
    """

    if not isinstance(objekt, str):
        return objekt
    return unicode(objekt, 'ascii')



class IndexableTextExtractor(object):

    wordssearch_rx = re.compile(r'\w+', re.UNICODE)
    text_extract_xpath = etree.XPath('text()')

    def __init__(self, content_type, *text_elements):
        """Building the extractor
        @param content_type: content_type of the part for which the extractor is defined
        @param text_elements: default text elements. See self.addTextElement(...)
        """

        self.content_type = content_type
        self.text_elts_xpaths = []
        for te in text_elements:
            self.addTextElement(te)
        return


    def addTextElement(self, element_name):
        """Adding an element that may contanin text to index
        @param element_name: an element that contains text to extract.
             the name may be prefixed with a key from namespaces.ns_map
        """

        self.text_elts_xpaths.append(etree.XPath('//' + element_name, ns_map))
        return


    def indexableText(self, tree):
        """Provides the indexable - search engine oriented - raw text
        @param tree: an ElementTreee
        @return: set(["foo", "bar", ...])"
        """

        rval = set()
        root = tree.getroot()
        for txp in self.text_elts_xpaths:
            elts = txp(root)
            texts = [self.text_extract_xpath(elt) for elt in elts]
            # Texts in element may be empty
            texts = [toUnicode(x)[0] for x in texts
                     if len(x) > 0]
            for text in texts:
                words = self.wordssearch_rx.findall(text)
                rval |= set(words)
        return rval




# -*- coding: utf-8 -*-
"""
Testing WordProcessingDocument
"""
# $Id: test_wordprocessing.py 6355 2007-09-20 17:16:21Z glenfant $

import unittest
import os
from fixures import *

import openxmllib

class WordProcessingTest(unittest.TestCase):
    """Testing querying properties from a document"""

    def setUp(self):
        test_file_path = os.path.join(TEST_FILES_IN, ALL_IN_FILES[0])
        self.doc = openxmllib.openXmlDocument(test_file_path)
        return


    def test_indexableText(self):
        """Indexable text with properties"""

        itext = self.doc.indexableText()
        some_words = (u'A', u'full', u'chàractèrs', u'non', u'custom_value_2', u'title')
        for word in some_words:
            self.failUnless(word in itext, "%s was expected" % word)
        return


    def test_indexableTextNoprop(self):
        """Indexable text without properties"""

        itext = self.doc.indexableText(include_properties=False)
        some_words = (u'A', u'full', u'chàractèrs', u'non')
        for word in some_words:
            self.failUnless(word in itext, "%s was expected" % word)
        return
# /class WordProcessingTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(WordProcessingTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



# -*- coding: utf-8 -*-
"""
The various inner content types in an open XML document
"""
# $Id: contenttypes.py 6800 2007-12-04 11:17:01Z glenfant $

import os
from lxml import etree
import namespaces as ns
import utils

# Common properties
CT_CORE_PROPS = 'application/vnd.openxmlformats-package.core-properties+xml'
CT_EXT_PROPS = 'application/vnd.openxmlformats-officedocument.extended-properties+xml'
CT_CUSTOM_PROPS = 'application/vnd.openxmlformats-officedocument.custom-properties+xml'

# Wordprocessing document
# See...
# http://technet2.microsoft.com/Office/en-us/library/e077da98-0216-45eb-b6a7-957f9c510a851033.mspx?pf=true
# ...for the various
CT_WORDPROC_DOCX_PUBLIC = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
CT_WORDPROC_DOTX_PUBLIC = 'application/vnd.openxmlformats-officedocument.wordprocessingml.template'
CT_WORDPROC_DOCM_PUBLIC = 'application/vnd.ms-word.document.macroEnabled.12'
CT_WORDPROC_DOTM_PUBLIC = 'application/vnd.ms-word.template.macroEnabled.12'

CT_WORDPROC_DOCUMENT = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml'
CT_WORDPROC_NUMBERING = 'application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml'
CT_WORDPROC_STYPES = 'application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml'
CT_WORDPROC_FONTS = 'application/vnd.openxmlformats-officedocument.wordprocessingml.fontTable+xml'
CT_WORDPROC_SETINGS = 'application/vnd.openxmlformats-officedocument.wordprocessingml.settings+xml'
CT_WORDPROC_FOOTNOTES = 'application/vnd.openxmlformats-officedocument.wordprocessingml.footnotes+xml'
CT_WORDPROC_ENDNOTES = 'application/vnd.openxmlformats-officedocument.wordprocessingml.endnotes+xml'
CT_WORDPROC_COMMENTS = 'application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml'

# Presentation document
CT_PRESENTATION_PPTX_PUBLIC = 'application/vnd.openxmlformats-officedocument.presentationml.presentation'
CT_PRESENTATION_PPTM_PUBLIC = 'application/vnd.ms-powerpoint.presentation.macroEnabled.12'
CT_PRESENTATION_PPSX_PUBLIC = 'application/vnd.openxmlformats-officedocument.presentationml.slideshow'
CT_PRESENTATION_PPSM_PUBLIC = 'application/vnd.ms-powerpoint.slideshow.macroEnabled.12'
CT_PRESENTATION_PPAM_PUBLIC = 'application/vnd.ms-powerpoint.addin.macroEnabled.12'
CT_PRESENTATION_POTX_PUBLIC = 'application/vnd.openxmlformats-officedocument.presentationml.template'
CT_PRESENTATION_POTM_PUBLIC = 'application/vnd.ms-powerpoint.template.macroEnabled.12'

# FIXME: Other presentation inner content types but useless for now...
CT_PRESENTATION_SLIDE = 'application/vnd.openxmlformats-officedocument.presentationml.slide+xml'

# Spreadsheet document
CT_SPREADSHEET_XLAM_PUBLIC = 'application/vnd.ms-excel.addin.macroEnabled.12'
CT_SPREADSHEET_XLSB_PUBLIC = 'application/vnd.ms-excel.sheet.binary.macroEnabled.12'
CT_SPREADSHEET_XLSM_PUBLIC = 'application/vnd.ms-excel.sheet.macroEnabled.12'
CT_SPREADSHEET_XLSX_PUBLIC = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
CT_SPREADSHEET_XLTM_PUBLIC = 'application/vnd.ms-excel.template.macroEnabled.12'
CT_SPREADSHEET_XLTX_PUBLIC = 'application/vnd.openxmlformats-officedocument.spreadsheetml.template'

# FIXME: Other spreadsheet inner content types but useless for now...
CT_SPREADSHEET_WORKSHEET = 'application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml'
CT_SPREADSHEET_SHAREDSTRINGS = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml'


class ContentTypes(object):
    """Handles features from the [Content_Types].xml file"""

    def __init__(self, content_types_file):
        """Constructor
        @param content_types_file: a file like object of [Content_Types].xml
        """

        self.overrides = {} # {subpart content type: [xml file, ...], ...}
        context = etree.iterparse(content_types_file, tag='{%s}Override' % ns.CONTENT_TYPES)
        for dummy, override in context:
            key = override.get('ContentType')
            if self.overrides.has_key(key):
                self.overrides[key].append(override.get('PartName'))
            else:
                self.overrides[key] = [override.get('PartName')]
        return


    def getPathsForContentType(self, content_type):
        """Finds the path in the document to that content type
        @param content_type: a MIME content type
        @return: list of paths in the content type
        """

        return self.overrides.get(content_type, [])


    def getTreesFor(self, document, content_type):
        """Provides all XML documents for that content type
        @param document: a Document or subclass object
        @param content_type: a MIME content type
        @return: list of etree._ElementTree of that content type
        """

        # Relative path without potential leading path separator
        # otherwise os.path.join doesn't work
        for rel_path in self.overrides[content_type]:
            if rel_path[0] in ('/', '\\'):
                rel_path = rel_path[1:]
            file_path = os.path.join(document._cache_dir, rel_path)
            yield etree.parse(utils.xmlFile(file_path, 'rb'))
        return


    @property
    def listMetaContentTypes(self):
        """The content types with metadata
        @return: ['application/xxx', ...]
        """

        all_md_content_types = (
            CT_CORE_PROPS,
            CT_EXT_PROPS,
            CT_CUSTOM_PROPS)
        return [k for k in self.overrides.keys() if k in all_md_content_types]




# -*- coding: utf-8 -*-
"""
The presentation module handles a PresentationML Open XML document (read *.pptx)
"""
# $Id: presentation.py 6800 2007-12-04 11:17:01Z glenfant $

import document
from utils import IndexableTextExtractor
import contenttypes as ct


class PresentationDocument(document.Document):
    """Handles specific features of a PresentationML document"""

    _extpattern_to_mime = {
        '*.pptx': ct.CT_PRESENTATION_PPTX_PUBLIC,
        '*.pptm': ct.CT_PRESENTATION_PPTM_PUBLIC,
        '*.potx': ct.CT_PRESENTATION_POTX_PUBLIC,
        '*.potm': ct.CT_PRESENTATION_POTM_PUBLIC,
        '*.ppsx': ct.CT_PRESENTATION_PPSX_PUBLIC,
        '*.ppsm': ct.CT_PRESENTATION_PPSM_PUBLIC,
        # FIXME: Not sure we can honour below types
#        '*.ppam': ct.CT_PRESENTATION_PPAM_PUBLIC
        }

    _text_extractors = (
        IndexableTextExtractor(ct.CT_PRESENTATION_SLIDE, 'presentation-main:t'),
        )
    pass



# -*- coding: utf-8 -*-
"""
The Open XML document library
Open XML document is defined by the ECMA-376 standard
http://www.ecma-international.org/publications/standards/Ecma-376.htm
"""
# $Id: __init__.py 6864 2007-12-06 14:09:09Z glenfant $

version_info = (1, 0, 1)
version = '.'.join([str(x) for x in version_info])

import os

import wordprocessing
import spreadsheet
import presentation

_document_classes = (
    wordprocessing.WordprocessingDocument,
    spreadsheet.SpreadsheetDocument,
    presentation.PresentationDocument)

def openXmlDocument(file_or_path_or_body, mime_type=None):
    """Factory function
    Will guess what document type is best suited and return the appropriate
    document type.
    # FIXME: this is very poor at the moment
    @param file_or_path_or_body: file object or path to file or file body.
    @param mime_type: mime type if known
    @return : Document subclass object
    Warning, when passing a file body, the mime_type is required
    """

    # Mime type based document
    if mime_type is not None:
        for klass in _document_classes:
            if klass.canProcessMime(mime_type):
                return klass(file_or_path_or_body, mime_type=mime_type)
        raise ValueError("%s MIME type is unknown." % mime_type)

    # File extension based factory
    if hasattr(file_or_path_or_body, 'read'):
        # File like object
        filename = file_or_path_or_body.name
    else:
        filename = os.path.basename(file_or_path_or_body)
    for klass in _document_classes:
        if klass.canProcessFilename(filename):
            return klass(file_or_path_or_body)
    raise ValueError("'%s' document cannot be processed here." % filename)




# -*- coding: utf-8 -*-
"""
Executes all tests here
"""
# $Id: runalltests.py 6355 2007-09-20 17:16:21Z glenfant $

import os
import unittest

TestRunner = unittest.TextTestRunner
suite = unittest.TestSuite()

this_dir = os.path.dirname(os.path.abspath(__file__))

testfiles = [n[:-3] for n in os.listdir(this_dir)
             if n.startswith('test') and n.endswith('.py')]

for testfile in testfiles:
    module = __import__(testfile)
    if hasattr(module, 'test_suite'):
        suite.addTest(module.test_suite())

if __name__ == '__main__':
    TestRunner().run(suite)



# -*- coding: utf-8 -*-
"""
Testing construction on valid and crappy files through the factory
"""
# $Id: test_factory.py 6857 2007-12-05 17:57:56Z glenfant $

import unittest
import os
from fixures import *

import openxmllib
import openxmllib.contenttypes as ct

class FactoryTest(unittest.TestCase):
    """Testing construction of Document various ways"""


    def test_frompath(self):
        """Construction from a path"""

        for test_filename in ALL_IN_FILES:
            test_filepath = os.path.join(TEST_FILES_IN, test_filename)
            doc = openxmllib.openXmlDocument(test_filepath)
            self.failUnless(isinstance(doc, openxmllib.document.Document),
                            "%s should be processed" % test_filepath)
        return

    def test_fromfile(self):
        """Construction from a file file object"""

        for test_filename in ALL_IN_FILES:
            test_filepath = os.path.join(TEST_FILES_IN, test_filename)
            fh = file(test_filepath, 'rb')
            doc = openxmllib.openXmlDocument(test_filepath)
            fh.close()
            self.failUnless(isinstance(doc, openxmllib.document.Document),
                            "%s should be processed" % test_filepath)
        return

    def test_frommime(self):
        """Construction with a mimetype help"""

        # From file path
        mime_type = ct.CT_WORDPROC_DOCX_PUBLIC
        test_file_path = os.path.join(TEST_FILES_IN, ALL_IN_FILES[0])
        doc = openxmllib.openXmlDocument(test_file_path, mime_type)
        self.failUnless(isinstance(doc, openxmllib.wordprocessing.WordprocessingDocument),
                        "Failed to create with mime type %s" % mime_type)
        self.failUnlessEqual(doc.mimeType, mime_type)

        # From file object
        fh = file(test_file_path, 'rb')
        doc = openxmllib.openXmlDocument(fh, mime_type)
        fh.close()
        self.failUnless(isinstance(doc, openxmllib.wordprocessing.WordprocessingDocument),
                        "Failed to create with mime type %s" % mime_type)
        self.failUnlessEqual(doc.mimeType, mime_type)

        # From file content
        fh = file(test_file_path, 'rb')
        doc = openxmllib.openXmlDocument(fh.read(), mime_type)
        fh.close()
        self.failUnless(isinstance(doc, openxmllib.wordprocessing.WordprocessingDocument),
                        "Failed to create with mime type %s" % mime_type)
        self.failUnlessEqual(doc.mimeType, mime_type)
        return
# /class FactoryTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FactoryTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



# -*- coding: utf-8 -*-
"""
$Id: setup.py 6864 2007-12-06 14:09:09Z glenfant $
"""

from setuptools import setup, find_packages
import os


def read(*rnames):
  return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


name             = 'openxmllib'
version          = '1.0.1'
long_description = "%s\n%s\n%s" % (read('README'), read('TODO'), read('HISTORY'))


setup( name                 = name
     , version              = version
     , description          = 'Provides resources to handle OpenXML documents from Python.'
     , long_description     = long_description
     , author               = 'Gilles Lenfant'
     , author_email         = 'gilles.lenfant@gmail.com'
     , url                  = 'http://code.google.com/p/openxmllib/'
     , license              = "GPLv2"
     , keywords             = 'Python OpenXML lxml Office2007 ECMA376'
     , classifiers          = [ "License :: OSI Approved :: GNU General Public License (GPL)"
                              , "Topic :: Office/Business :: Office Suites"
                              , "Topic :: Software Development :: Libraries :: Python Modules"
                              , "Topic :: Utilities"
                              , "Programming Language :: Python"
                              ]
     , packages             = find_packages(name, exclude='tests')
     , package_dir          = {'': name}
     , include_package_data = True
     , install_requires     = ['lxml>=1.3.0,<2.0.0dev', ]
     , zip_safe             = False
     , scripts              = ['scripts/openxmlinfo.py', ]
     )




# -*- coding: utf-8 -*-
"""
Testing ContentTypes object and services
"""
# $Id: test_contenttypes.py 6796 2007-12-04 10:52:51Z glenfant $

import unittest
import StringIO

from fixures import *

import openxmllib.namespaces as ns
import openxmllib.contenttypes as ct


CONTENTTYPES_XML = """<?xml version="1.0" encoding="utf-8"?>
<Types xmlns="%(content-types)s">
  <Default Extension="jpeg" ContentType="image/jpeg" />
  <Default Extension="jpg" ContentType="image/jpeg" />
  <Default Extension="gif" ContentType="image/gif" />
  <Default Extension="png" ContentType="image/png" />
  <Default Extension="wmf" ContentType="image/x-emf" />
  <Default Extension="emf" ContentType="image/x-emf" />
  <Default Extension="tif" ContentType="image/tiff" />
  <Default Extension="tiff" ContentType="image/tiff" />
  <Default Extension="bin"
           ContentType="application/vnd.openxmlformats-officedocument.oleObject" />
  <Default Extension="rels"
           ContentType="application/vnd.openxmlformats-package.relationships+xml" />
  <Default Extension="xml" ContentType="application/xml" />
  <Override PartName="/word/document.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml" />
  <Override PartName="/word/numbering.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml" />
  <Override PartName="/docProps/core.xml"
            ContentType="application/vnd.openxmlformats-package.core-properties+xml" />
  <Override PartName="/docProps/app.xml"
            ContentType="application/vnd.openxmlformats-officedocument.extended-properties+xml" />
  <Override PartName="/docProps/custom.xml"
            ContentType="application/vnd.openxmlformats-officedocument.custom-properties+xml" />
  <Override PartName="/word/styles.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml" />
  <Override PartName="/word/fontTable.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.fontTable+xml" />
  <Override PartName="/word/settings.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.settings+xml" />
  <Override PartName="/word/footnotes.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.footnotes+xml" />
  <Override PartName="/word/endnotes.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.endnotes+xml" />
  <Override PartName="/word/comments.xml"
            ContentType="application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml" />
</Types>
""" % ns.ns_map


class ContentTypesTest(unittest.TestCase):
    """Testing the ContentTypes object"""


    def setUp(self):
        self.content_types = ct.ContentTypes(StringIO.StringIO(CONTENTTYPES_XML))
        return

    def test_overrides(self):
        """Checking some overrides"""

        self.failUnlessEqual(self.content_types.getPathsForContentType(ct.CT_CORE_PROPS),
                             ['/docProps/core.xml'])
        self.failUnlessEqual(self.content_types.getPathsForContentType(ct.CT_EXT_PROPS),
                             ['/docProps/app.xml'])
        self.failUnlessEqual(self.content_types.getPathsForContentType(ct.CT_CUSTOM_PROPS),
                             ['/docProps/custom.xml'])
        return
# /class ContentTypesTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ContentTypesTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



# -*- coding: utf-8 -*-
"""
Testing the fixures
"""
# $Id: test_fixures.py 6796 2007-12-04 10:52:51Z glenfant $

import unittest
import sys
import os
import fixures

class FixuresTest(unittest.TestCase):
    """Testing the tests environment"""

    def test_syspath(self):
        """Do we include the openxmllib package in sys.path"""

        self.failUnlessEqual(fixures.parent_dir, sys.path[0],
                             "%s not in sys.path" % fixures.parent_dir)
        sp_copy = list(sys.path)
        sp_copy.remove(fixures.parent_dir)
        self.failIf(fixures.parent_dir in sp_copy,
                    "%s should appear only once in sys.path" % fixures.parent_dir)
        return


    def test_testdir(self):
        """Do we have test data"""

        self.failUnless(os.path.isdir(fixures.TEST_FILES_IN),
                        "We have no test data")
        return

    def test_import(self):
        """Do we import our lovely package"""

        import openxmllib
        dummy = openxmllib.openXmlDocument
        return
# /class FixuresTest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FixuresTest))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())
